import collections
import json
import glob
import os
import csv

import numpy
import numpy as np
from nltk import word_tokenize


# Code here

# Read data
def read_json(path):
    with open(path) as f:
        data = json.load(f)
    return data


def read_score(path):
    with open(path) as f:
        r = csv.DictReader(f)
        data = dict()
        for row in r:
            data[row['PST_ID']] = int(row['FinalScore'])
    return data


def read_coding_data():
    all_data = []
    for file in glob.glob('data/ets-annotation-webapp-main/static/data/week*/*/*.json'):
        data = read_json(file)
        all_data.append(data)  # Duy added 4/7/2022: randomly sample one transcript for each pair of double-scored
                               # transcripts
                               # Duy added 4/12/2022: Replace transcripts with the reconciled one
    return all_data


def mapping_score_data():
    scores = read_score('data/Transcript Scores.csv')
    print(scores)

    codding_data = read_coding_data()
    for i, x in enumerate(codding_data):
        tid = x['title'][:7]
        score = scores[tid]
        codding_data[i]['score'] = score
        codding_data[i]['feature'] = extract_frequency_feature(x) #+ extract_bigram_code_feature(x)
        codding_data[i]['PST_ID'] = tid  # Duy added
    return codding_data


def extract_frequency_feature(transcript):
    annotation = transcript['annotation']
    counter = collections.Counter()
    for utterance in annotation:
        counter.update(utterance['utterance_label'])
    labels = ['BCS', 'ELC', 'ELD', 'ELR', 'EVL', 'EXA', 'PVD', 'PVR', 'STC']
    frequencies = [counter[l] for l in labels]
    total = sum(frequencies)
    if total == 0:
        props = [1.0 / 9 for _ in labels] # Duy added 4/7: what does this line of code do?
    else:
        props = [x / total for x in frequencies]
    return props

def extract_bigram_code_feature(transcript):

    labels = ['BCS', 'ELC', 'ELD', 'ELR', 'EVL', 'EXA', 'PVD', 'PVR', 'STC']
    pairs = [(x,y) for x, y in zip(labels, labels)]

    utterance_labels = [x['utterance_label'] for x in transcript['annotation']]
    all_pairs = []
    for uid in range(len(utterance_labels)-1):
        current_labels = utterance_labels[uid]
        next_labels = utterance_labels[uid+1]
        for l1 in current_labels:
            for l2 in next_labels:
                all_pairs.append((l1, l2))
    counter = collections.Counter()
    counter.update(all_pairs)

    frequencies =[]

    for x  in pairs:
        if x in counter:
            frequencies.append(counter[x])
        else:
            frequencies.append(0)

    total = len(all_pairs)
    if total == 0:
        props = [1.0 / len(pairs) for _ in labels] # Duy added 4/7: what does this line of code do?
    else:
        props = [x / total for x in frequencies]
    return props

def extract_word_feature(transcript):
    # Dictionary
    with open ("resources/vocab.txt") as f:
        words = f.read().split('\n')
        vocab ={
            w: i
            for i, w in enumerate(words)

        }

    # Loop
    vect = numpy.zeros((len(vocab)))
    for utterance in transcript["annotation"]:
        sentences = utterance["text"]
        for sent in sentences:
            tokens = word_tokenize(sent.lower())
            for token in tokens:
                if token in vocab:
                    vect[vocab[token]]=1.0
    return vect





from sklearn.linear_model import LinearRegression

if __name__ == '__main__':

    coding_data = mapping_score_data()

    features = [x['feature'] for x in coding_data]
    scores = [x['score'] for x in coding_data]
    tid = [x['PST_ID'] for x in coding_data]  # Duy added to print Transcript ID out
    model = LinearRegression()


    model.fit(features, scores)

    predictions = model.predict(features)

    with open('results/predictions.csv', 'w') as f:
        for i, p, s in zip(tid, predictions, scores): #  Duy added i to print out PST_ID
            # print(i, p, s)
            f.write(f'{i}, {p},{s}\n')

# Duy's Notes:
# 1. We should only use data from week1 onward to week15
    # 1.1. We should incorporate the reconciled data into the transcripts before building the model
# 2. We should randomly select one of each the double-score transcript pairs to build the models (KISS)
# 3. For feature-based models, we should try to use features for (i) all utterances (i.e., PST and students), and (ii)
  # features for PST's utterances only
# 4. How will we select 70 training and 30 testing transcripts? Should they be both representative of the 100 selected one?
    #4.1 # Divide the 100 transcripts into five pars (P1, P2, P3, P4, P5).
        # P1 - P4: Training set, P5 will the testing set.
    #4.2. Cross validation, permute training set and testing sets
# 5. Should we only apply the prediction model to the testing set, instead of the all the transcripts
    # (i.e., including the one we used to build the regression model)
# 6. We should also write code for the deep learning model without using any annotated data

# Questions about Deep Learning
    # Are all activation values between 0 and 1? it depends on the activation function. 
    # Why do we need the bias term in the weighted sum of neurons?
    # Why is ReLU function easier to be trained than the Sigmoid or Tangh functions? ReLU(a) = max(o,a) which can be > 1
    # In the recurrent neural network, when to forget what?
    # How to augument data to generate more training set?
    # Data types and how to know the types of an object
    # Operations we can do with each type of data
    # View structure of an object, view the data in table format
    # Reference elements within an object
    # Can we use linear SVM when the predicted scores have more than 2 categories (e.g., 0, 1, 2)?
    # Can we use non-linear SVM?

# scores = read_score('data/Transcript Scores.csv')
# sum(abs(predictions - int(scores)))
# import pandas'

    print(predictions)

    predictions = np.array(predictions)
    scores = np.array(scores)

    diff = scores - predictions
    abs_diff = np.sum(np.abs(diff))
    mse = np.sum(diff * diff)  # Duy added 4/12: should be divided by the number of transcripts to get the mean,
                               # Then, take the square root to put the rmse on the original scale

    print('Absoluate: ', abs_diff)
    print('MSE: ', mse)

# 1 Word features
  # One-hot - (Support Vector Machine


# with open('resources/vocab.txt') as f:
#     words = f.readlines()
#     words = [x.strip() for x in words]
#
# word2index = {
#     word: i
#     for i, word in enumerate(words)
# }
#
# vocab_size = len(word2index)
#
#
# sentence = 'This is a sample sentence'
#
# tokens = sentence.lower().split(' ')
# tokens = [x for x in tokens if len(x) >0]
#
# sentence_vectors = []
# for token in tokens:
#     vector = numpy.zeros((vocab_size))
#     if token in word2index:
#         index = word2index[token]
#         vector[index] = 1.0
#     sentence_vectors.append(vector)

# 2 Frequency features
  # float
# 3 Flow features
  # Pairs of features 100 feature pairs if we have 10 codes - (e.g., 2 or n-grams - across features)



# Within utterence features

