import glob
import os
import socket

# hostname = socket.gethostname()
hostname = 'ip-10-216-143-38'

DERBERTA = 'microsoft/deberta-v3-large'
ENGLISH = 'en'

from itertools import product

DEFAULT_LRS = [0.00002, 0.00005]


def schedule_baseline():
    LMS = ['bert-base-cased']
    # LMS = ['bert-large-cased']
    LRS = DEFAULT_LRS
    optimizer = 'adamw'
    model = 'lstm'
    BATCH_SIZES = [1,2,3,4]
    utterance = 8
    ACC_ITERS = [1]

    configs = product(BATCH_SIZES, ACC_ITERS, LMS, LRS)

    f = open('commands.txt', 'a')
    for bs, acc_iter, lm, lr in configs:
        epoch = 5 * acc_iter
        name = f'{model}/{model}.{lm}.bs-{bs}.ut-{utterance}.optim-{optimizer}.lr-{lr}'
        os.makedirs(f'logs/{name}', exist_ok=True)
        logfile = f'logs/{name}/log.txt'
        errorfile = f'logs/{name}/error.txt'
        command = 'python train.py' + \
                  f' --lm={lm}' + \
                  f' --batch_size={bs}' + \
                  f' --utterance={utterance}' + \
                  f' --model={model}' + \
                  f' --optimizer={optimizer}' + \
                  f' --lr={lr}' + \
                  f' --epoch={epoch}' + \
                  f' --acc_iter={acc_iter}' + \
                  f' > {logfile} 2> {errorfile}'
        f.write(command + '\n')
    f.close()


def schedule_trans():
    LMS = ['bert-base-cased']
    # LMS = ['bert-large-cased']
    LRS = DEFAULT_LRS
    optimizer = 'adamw'
    model = 'gcntrans'
    BATCH_SIZES = [1,2,3,4]
    utterance = 8
    ACC_ITERS = [1]

    configs = product(BATCH_SIZES, ACC_ITERS, LMS, LRS)

    f = open('commands.txt', 'a')
    for bs, acc_iter, lm, lr in configs:
        epoch = 5 * acc_iter
        name = f'{model}/{model}.{lm}.bs-{bs}.ut-{utterance}.optim-{optimizer}.lr-{lr}'
        os.makedirs(f'logs/{name}', exist_ok=True)
        logfile = f'logs/{name}/log.txt'
        errorfile = f'logs/{name}/error.txt'
        command = 'python train.py' + \
                  f' --lm={lm}' + \
                  f' --batch_size={bs}' + \
                  f' --utterance={utterance}' + \
                  f' --model={model}' + \
                  f' --optimizer={optimizer}' + \
                  f' --lr={lr}' + \
                  f' --epoch={epoch}' + \
                  f' --acc_iter={acc_iter}' + \
                  f' > {logfile} 2> {errorfile}'
        f.write(command + '\n')

def schedule_gcn():
    model = 'gcnlstm'
    LMS = ['bert-base-cased']
    # LMS = ['bert-large-cased']
    LRS = DEFAULT_LRS
    optimizer = 'adam'
    BATCH_SIZES = [4]
    utterance = 8
    ACC_ITERS = [1]

    configs = product(BATCH_SIZES, ACC_ITERS, LMS, LRS)

    f = open('commands.txt', 'a')
    for bs, acc_iter, lm, lr in configs:
        epoch = 10 * acc_iter
        name = f'{model}/{model}.{lm}.bs-{bs}.ut-{utterance}.optim-{optimizer}.lr-{lr}'
        os.makedirs(f'logs/{name}', exist_ok=True)
        logfile = f'logs/{name}/log.txt'
        errorfile = f'logs/{name}/error.txt'
        command = 'python train.py' + \
                  f' --lm={lm}' + \
                  f' --batch_size={bs}' + \
                  f' --utterance={utterance}' + \
                  f' --model={model}' + \
                  f' --optimizer={optimizer}' + \
                  f' --lr={lr}' + \
                  f' --epoch={epoch}' + \
                  f' --acc_iter={acc_iter}' + \
                  f' > {logfile} 2> {errorfile}'
        f.write(command + '\n')
    f.close()


def schedule_cnn(model):
    LMS = ['bert-base-cased']
    # LMS = ['bert-large-cased']
    LRS = DEFAULT_LRS
    optimizer = 'adam'
    BATCH_SIZES = [4]
    CNN_KERNEL_SIZES = [1,2,3,4]
    utterance = 8
    ACC_ITERS = [1]

    configs = product(CNN_KERNEL_SIZES, BATCH_SIZES, ACC_ITERS, LMS, LRS)

    f = open('commands.txt', 'a')
    for kernel, bs, acc_iter, lm, lr in configs:
        epoch = 10 * acc_iter
        name = f'{model}/{model}.{lm}.bs-{bs}.ut-{utterance}.kernel-{kernel}.optim-{optimizer}.lr-{lr}'
        os.makedirs(f'logs/{name}', exist_ok=True)
        logfile = f'logs/{name}/log.txt'
        errorfile = f'logs/{name}/error.txt'
        command = 'python train.py' + \
                  f' --lm={lm}' + \
                  f' --batch_size={bs}' + \
                  f' --utterance={utterance}' + \
                  f' --model={model}' + \
                  f' --optimizer={optimizer}' + \
                  f' --lr={lr}' + \
                  f' --epoch={epoch}' + \
                  f' --acc_iter={acc_iter}' + \
                  f' --kernel_size={kernel}' + \
                  f' > {logfile} 2> {errorfile}'
        f.write(command + '\n')
    f.close()


if __name__ == '__main__':
    # schedule_baseline()
    # schedule_cnn('cnn')
    # schedule_cnn('gcncnn')
    # schedule_cnn('cnnlstm')
    # schedule_gcn()

    schedule_trans()
    with open('commands.txt') as f:
        commands = f.read()
    print(commands)
