import json
import numpy as np
from numpy.random import random
import os
import glob
from sklearn.metrics import precision_score, recall_score, f1_score, cohen_kappa_score
from transformers import *
import torch
LABEL_SET = ['BCS', 'ELC', 'ELD', 'ELR', 'EVL', 'EXA', 'PVD', 'PVR', 'STC']


def read_json(path):
    with open(path) as f:
        data = json.load(f)
    return data


def save_json(data, path):
    dir_path = os.path.dirname(path)
    os.makedirs(dir_path, exist_ok=True)
    with open(path, 'w') as f:
        json.dump(data, f, indent=2)


if __name__ == '__main__':

    tokenizer = AutoTokenizer.from_pretrained('bert-base-cased')
    model = AutoModel.from_pretrained('bert-base-cased')
    # model.cuda()

    all_files = glob.glob('data/splits/*/*.json')

    feature_dict = dict()

    for file in all_files:
        filename = os.path.basename(file)
        json_obj = read_json(file)
        annotation_data = json_obj['annotation']

        all_texts = []
        for utterance in annotation_data:
            text = '[CLS] ' + ' [SEP] '.join(utterance['text']) + ' [SEP]'
            all_texts.append(text)

        inputs = tokenizer(all_texts, return_tensors='pt', truncation=True, padding=True)

        # for k, v in inputs.items():
            # v.cuda()

        outputs = model(**inputs, return_dict=True)

        last_hidden_state = outputs.last_hidden_state
        print(last_hidden_state.shape)
        cls = last_hidden_state[:,0,:]

        print(cls.shape)
        features = torch.mean(cls,dim=0)

        print(features.shape)
        features = features.detach().cpu().tolist()
        feature_dict[filename] = features

    save_json('resources/bert-base-cased.json')